@@ -33,7 +33,7 @@ import re
 import unittest
 from binascii import hexlify, unhexlify
 
-from Cryptodome.Util.py3compat import b, tobytes, bchr, _memoryview
+from Cryptodome.Util.py3compat import b, tobytes, bchr
 from Cryptodome.Util.strxor import strxor_c
 from Cryptodome.SelfTest.st_common import list_test_cases
 
@@ -58,9 +58,20 @@ class ChaCha20Test(unittest.TestCase):
     def test_default_nonce(self):
         cipher1 = ChaCha20.new(key=bchr(1) * 32)
         cipher2 = ChaCha20.new(key=bchr(1) * 32)
-        self.assertEquals(len(cipher1.nonce), 8)
+        self.assertEqual(len(cipher1.nonce), 8)
         self.assertNotEqual(cipher1.nonce, cipher2.nonce)
 
+    def test_nonce(self):
+        key = b'A' * 32
+
+        nonce1 = b'P' * 8
+        cipher1 = ChaCha20.new(key=key, nonce=nonce1)
+        self.assertEqual(nonce1, cipher1.nonce)
+
+        nonce2 = b'Q' * 12
+        cipher2 = ChaCha20.new(key=key, nonce=nonce2)
+        self.assertEqual(nonce2, cipher2.nonce)
+
     def test_eiter_encrypt_or_decrypt(self):
         """Verify that a cipher cannot be used for both decrypting and encrypting"""
 
@@ -117,7 +128,7 @@ class ChaCha20Test(unittest.TestCase):
         cipher2.seek(offset)
         ct2 = cipher2.encrypt(pt)
 
-        self.assertEquals(ct1, ct2)
+        self.assertEqual(ct1, ct2)
 
     def test_seek_tv(self):
         # Test Vector #4, A.1 from
@@ -265,6 +276,12 @@ class XChaCha20Test(unittest.TestCase):
 
         self.assertEqual(subkey, expected)
 
+    def test_nonce(self):
+        key = b'A' * 32
+        nonce = b'P' * 24
+        cipher = ChaCha20.new(key=key, nonce=nonce)
+        self.assertEqual(nonce, cipher.nonce)
+
     def test_encrypt(self):
         # Section A.3.2
 
@@ -456,42 +473,40 @@ class TestOutput(unittest.TestCase):
         nonce = b'5' * 8
         cipher = ChaCha20.new(key=key, nonce=nonce)
 
-        pt = b'5' * 16
+        pt = b'5' * 300
         ct = cipher.encrypt(pt)
 
-        output = bytearray(16)
+        output = bytearray(len(pt))
         cipher = ChaCha20.new(key=key, nonce=nonce)
         res = cipher.encrypt(pt, output=output)
         self.assertEqual(ct, output)
         self.assertEqual(res, None)
-        
+
         cipher = ChaCha20.new(key=key, nonce=nonce)
         res = cipher.decrypt(ct, output=output)
         self.assertEqual(pt, output)
         self.assertEqual(res, None)
 
-        import sys
-        if sys.version[:3] != '2.6':
-            output = memoryview(bytearray(16))
-            cipher = ChaCha20.new(key=key, nonce=nonce)
-            cipher.encrypt(pt, output=output)
-            self.assertEqual(ct, output)
-        
-            cipher = ChaCha20.new(key=key, nonce=nonce)
-            cipher.decrypt(ct, output=output)
-            self.assertEqual(pt, output)
+        output = memoryview(bytearray(len(pt)))
+        cipher = ChaCha20.new(key=key, nonce=nonce)
+        cipher.encrypt(pt, output=output)
+        self.assertEqual(ct, output)
 
         cipher = ChaCha20.new(key=key, nonce=nonce)
-        self.assertRaises(TypeError, cipher.encrypt, pt, output=b'0'*16)
-        
+        cipher.decrypt(ct, output=output)
+        self.assertEqual(pt, output)
+
         cipher = ChaCha20.new(key=key, nonce=nonce)
-        self.assertRaises(TypeError, cipher.decrypt, ct, output=b'0'*16)
+        self.assertRaises(TypeError, cipher.encrypt, pt, output=b'0'*len(pt))
+
+        cipher = ChaCha20.new(key=key, nonce=nonce)
+        self.assertRaises(TypeError, cipher.decrypt, ct, output=b'0'*len(pt))
+
+        shorter_output = bytearray(len(pt) - 1)
 
-        shorter_output = bytearray(7)
-        
         cipher = ChaCha20.new(key=key, nonce=nonce)
         self.assertRaises(ValueError, cipher.encrypt, pt, output=shorter_output)
-        
+
         cipher = ChaCha20.new(key=key, nonce=nonce)
         self.assertRaises(ValueError, cipher.decrypt, ct, output=shorter_output)
 
@@ -502,11 +517,7 @@ def get_tests(config={}):
     tests += list_test_cases(XChaCha20Test)
     tests.append(ChaCha20_AGL_NIR())
     tests.append(ByteArrayTest())
-
-    import sys
-    if sys.version[:3] != "2.6":
-        tests.append(MemoryviewTest())
-
+    tests.append(MemoryviewTest())
     tests.append(TestOutput())
 
     return tests
