@@ -85,7 +85,7 @@ class ChaCha20Poly1305Tests(unittest.TestCase):
 
         cipher = ChaCha20_Poly1305.new(key=self.key_256,
                                        nonce=self.nonce_96)
-        self.assertEquals(ct, cipher.encrypt(self.data_128))
+        self.assertEqual(ct, cipher.encrypt(self.data_128))
 
     def test_nonce_must_be_bytes(self):
         self.assertRaises(TypeError,
@@ -108,7 +108,7 @@ class ChaCha20Poly1305Tests(unittest.TestCase):
         # Not based on block ciphers
         cipher = ChaCha20_Poly1305.new(key=self.key_256,
                                        nonce=self.nonce_96)
-        self.failIf(hasattr(cipher, 'block_size'))
+        self.assertFalse(hasattr(cipher, 'block_size'))
 
     def test_nonce_attribute(self):
         cipher = ChaCha20_Poly1305.new(key=self.key_256,
@@ -225,7 +225,7 @@ class ChaCha20Poly1305Tests(unittest.TestCase):
             for chunk in break_up(plaintext, chunk_length):
                 ct2 += cipher.encrypt(chunk)
             self.assertEqual(ciphertext, ct2)
-            self.assertEquals(cipher.digest(), ref_mac)
+            self.assertEqual(cipher.digest(), ref_mac)
 
     def test_bytearray(self):
 
