@@ -28,22 +28,55 @@
 # POSSIBILITY OF SUCH DAMAGE.
 # ===================================================================
 
-import sys
+import os
+import errno
+import warnings
 import unittest
 from binascii import unhexlify
 
 from Cryptodome.SelfTest.st_common import list_test_cases
-from Cryptodome.Util._file_system import pycryptodome_filename
-from Cryptodome.Util.py3compat import bord, tostr
+from Cryptodome.Util.py3compat import bord, tostr, FileNotFoundError
+from Cryptodome.Util.asn1 import DerSequence, DerBitString
 from Cryptodome.Util.number import bytes_to_long
 from Cryptodome.Hash import SHAKE128
 
 from Cryptodome.PublicKey import ECC
 
-def load_file(filename, mode="rb"):
-    comps = [ "Cryptodome", "SelfTest", "PublicKey", "test_vectors", "ECC" ]
-    with open(pycryptodome_filename(comps, filename), mode) as fd:
-        return fd.read()
+try:
+    import pycryptodome_test_vectors  # type: ignore
+    test_vectors_available = True
+except ImportError:
+    test_vectors_available = False
+
+
+class MissingTestVectorException(ValueError):
+    pass
+
+
+def load_file(file_name, mode="rb"):
+    results = None
+
+    try:
+        if not test_vectors_available:
+            raise FileNotFoundError(errno.ENOENT,
+                                    os.strerror(errno.ENOENT),
+                                    file_name)
+
+        dir_comps = ("PublicKey", "ECC")
+        init_dir = os.path.dirname(pycryptodome_test_vectors.__file__)
+        full_file_name = os.path.join(os.path.join(init_dir, *dir_comps), file_name)
+        with open(full_file_name, mode) as file_in:
+            results = file_in.read()
+
+    except FileNotFoundError:
+        warnings.warn("Warning: skipping extended tests for ECC",
+                      UserWarning,
+                      stacklevel=2)
+
+    if results is None:
+        raise MissingTestVectorException("Missing %s" % file_name)
+
+    return results
 
 
 def compact(lines):
@@ -87,13 +120,19 @@ def create_ref_keys_p521():
     return (ECC.construct(curve="P-521", d=private_key_d),
             ECC.construct(curve="P-521", point_x=public_key_x, point_y=public_key_y))
 
-
 # Create reference key pair
 # ref_private, ref_public = create_ref_keys_p521()
 
-
 def get_fixed_prng():
-        return SHAKE128.new().update(b"SEED").read
+    return SHAKE128.new().update(b"SEED").read
+
+
+def extract_bitstring_from_spki(data):
+        seq = DerSequence()
+        seq.decode(data)
+        bs = DerBitString()
+        bs.decode(seq[1])
+        return bs.value
 
 
 class TestImport(unittest.TestCase):
@@ -104,7 +143,9 @@ class TestImport(unittest.TestCase):
 
 class TestImport_P256(unittest.TestCase):
 
-    ref_private, ref_public = create_ref_keys_p256()
+    def __init__(self, *args, **kwargs):
+        super(TestImport_P256, self).__init__(*args, **kwargs)
+        self.ref_private, self.ref_public = create_ref_keys_p256()
 
     def test_import_public_der(self):
         key_file = load_file("ecc_p256_public.der")
@@ -118,6 +159,18 @@ class TestImport_P256(unittest.TestCase):
         key = ECC.import_key(key_file)
         self.assertEqual(self.ref_public, key)
 
+    def test_import_sec1_uncompressed(self):
+        key_file = load_file("ecc_p256_public.der")
+        value = extract_bitstring_from_spki(key_file)
+        key = ECC.import_key(key_file, curve_name='P256')
+        self.assertEqual(self.ref_public, key)
+
+    def test_import_sec1_compressed(self):
+        key_file = load_file("ecc_p256_public_compressed.der")
+        value = extract_bitstring_from_spki(key_file)
+        key = ECC.import_key(key_file, curve_name='P256')
+        self.assertEqual(self.ref_public, key)
+
     def test_import_private_der(self):
         key_file = load_file("ecc_p256_private.der")
 
@@ -182,8 +235,6 @@ class TestImport_P256(unittest.TestCase):
         self.assertEqual(self.ref_private, key)
 
     def test_import_private_pem_with_ecparams(self):
-        if sys.version_info[:2] == (2, 6):
-            return
         key_file = load_file("ecc_p256_private_ecparams.pem")
         key = ECC.import_key(key_file)
         # We just check if the import succeeds
@@ -232,7 +283,9 @@ class TestImport_P256(unittest.TestCase):
 
 class TestImport_P384(unittest.TestCase):
 
-    ref_private, ref_public = create_ref_keys_p384()
+    def __init__(self, *args, **kwargs):
+        super(TestImport_P384, self).__init__(*args, **kwargs)
+        self.ref_private, self.ref_public = create_ref_keys_p384()
 
     def test_import_public_der(self):
         key_file = load_file("ecc_p384_public.der")
@@ -246,6 +299,18 @@ class TestImport_P384(unittest.TestCase):
         key = ECC.import_key(key_file)
         self.assertEqual(self.ref_public, key)
 
+    def test_import_sec1_uncompressed(self):
+        key_file = load_file("ecc_p384_public.der")
+        value = extract_bitstring_from_spki(key_file)
+        key = ECC.import_key(key_file, curve_name='P384')
+        self.assertEqual(self.ref_public, key)
+
+    def test_import_sec1_compressed(self):
+        key_file = load_file("ecc_p384_public_compressed.der")
+        value = extract_bitstring_from_spki(key_file)
+        key = ECC.import_key(key_file, curve_name='P384')
+        self.assertEqual(self.ref_public, key)
+
     def test_import_private_der(self):
         key_file = load_file("ecc_p384_private.der")
 
@@ -353,7 +418,9 @@ class TestImport_P384(unittest.TestCase):
 
 class TestImport_P521(unittest.TestCase):
 
-    ref_private, ref_public = create_ref_keys_p521()
+    def __init__(self, *args, **kwargs):
+        super(TestImport_P521, self).__init__(*args, **kwargs)
+        self.ref_private, self.ref_public = create_ref_keys_p521()
 
     def test_import_public_der(self):
         key_file = load_file("ecc_p521_public.der")
@@ -367,6 +434,18 @@ class TestImport_P521(unittest.TestCase):
         key = ECC.import_key(key_file)
         self.assertEqual(self.ref_public, key)
 
+    def test_import_sec1_uncompressed(self):
+        key_file = load_file("ecc_p521_public.der")
+        value = extract_bitstring_from_spki(key_file)
+        key = ECC.import_key(key_file, curve_name='P521')
+        self.assertEqual(self.ref_public, key)
+
+    def test_import_sec1_compressed(self):
+        key_file = load_file("ecc_p521_public_compressed.der")
+        value = extract_bitstring_from_spki(key_file)
+        key = ECC.import_key(key_file, curve_name='P521')
+        self.assertEqual(self.ref_public, key)
+
     def test_import_private_der(self):
         key_file = load_file("ecc_p521_private.der")
 
@@ -474,7 +553,9 @@ class TestImport_P521(unittest.TestCase):
 
 class TestExport_P256(unittest.TestCase):
 
-    ref_private, ref_public = create_ref_keys_p256()
+    def __init__(self, *args, **kwargs):
+        super(TestExport_P256, self).__init__(*args, **kwargs)
+        self.ref_private, self.ref_public = create_ref_keys_p256()
 
     def test_export_public_der_uncompressed(self):
         key_file = load_file("ecc_p256_public.der")
@@ -496,6 +577,21 @@ class TestExport_P256(unittest.TestCase):
         key_file_compressed_ref = load_file("ecc_p256_public_compressed.der")
         self.assertEqual(key_file_compressed, key_file_compressed_ref)
 
+    def test_export_public_sec1_uncompressed(self):
+        key_file = load_file("ecc_p256_public.der")
+        value = extract_bitstring_from_spki(key_file)
+
+        encoded = self.ref_public.export_key(format="SEC1")
+        self.assertEqual(value, encoded)
+
+    def test_export_public_sec1_compressed(self):
+        key_file = load_file("ecc_p256_public.der")
+        encoded = self.ref_public.export_key(format="SEC1", compress=True)
+
+        key_file_compressed_ref = load_file("ecc_p256_public_compressed.der")
+        value = extract_bitstring_from_spki(key_file_compressed_ref)
+        self.assertEqual(value, encoded)
+
     def test_export_private_der(self):
         key_file = load_file("ecc_p256_private.der")
 
@@ -520,7 +616,7 @@ class TestExport_P256(unittest.TestCase):
 
     def test_export_private_pkcs8_encrypted(self):
         encoded = self.ref_private._export_pkcs8(passphrase="secret",
-                                            protection="PBKDF2WithHMAC-SHA1AndAES128-CBC")
+                                                    protection="PBKDF2WithHMAC-SHA1AndAES128-CBC")
 
         # This should prove that the output is password-protected
         self.assertRaises(ValueError, ECC._import_pkcs8, encoded, None)
@@ -531,8 +627,8 @@ class TestExport_P256(unittest.TestCase):
         # ---
 
         encoded = self.ref_private.export_key(format="DER",
-                                         passphrase="secret",
-                                         protection="PBKDF2WithHMAC-SHA1AndAES128-CBC")
+                                                passphrase="secret",
+                                                protection="PBKDF2WithHMAC-SHA1AndAES128-CBC")
         decoded = ECC.import_key(encoded, "secret")
         self.assertEqual(self.ref_private, decoded)
 
@@ -626,15 +722,15 @@ class TestExport_P256(unittest.TestCase):
         key_file = load_file("ecc_p256_public_openssh.txt", "rt")
 
         encoded = self.ref_public._export_openssh(False)
-        self.assertEquals(key_file, encoded)
+        self.assertEqual(key_file, encoded)
 
         # ---
 
         encoded = self.ref_public.export_key(format="OpenSSH")
-        self.assertEquals(key_file, encoded)
+        self.assertEqual(key_file, encoded)
 
         encoded = self.ref_public.export_key(format="OpenSSH", compress=False)
-        self.assertEquals(key_file, encoded)
+        self.assertEqual(key_file, encoded)
 
     def test_export_openssh_compressed(self):
         key_file = load_file("ecc_p256_public_openssh.txt", "rt")
@@ -642,7 +738,7 @@ class TestExport_P256(unittest.TestCase):
 
         key_file_compressed = pub_key.export_key(format="OpenSSH", compress=True)
         assert len(key_file) > len(key_file_compressed)
-        self.assertEquals(pub_key, ECC.import_key(key_file_compressed))
+        self.assertEqual(pub_key, ECC.import_key(key_file_compressed))
 
     def test_prng(self):
         # Test that password-protected containers use the provided PRNG
@@ -654,7 +750,7 @@ class TestExport_P256(unittest.TestCase):
                                           passphrase="secret",
                                           protection="PBKDF2WithHMAC-SHA1AndAES128-CBC",
                                           randfunc=get_fixed_prng())
-        self.assertEquals(encoded1, encoded2)
+        self.assertEqual(encoded1, encoded2)
 
         # ---
 
@@ -666,7 +762,7 @@ class TestExport_P256(unittest.TestCase):
                                           use_pkcs8=False,
                                           passphrase="secret",
                                           randfunc=get_fixed_prng())
-        self.assertEquals(encoded1, encoded2)
+        self.assertEqual(encoded1, encoded2)
 
     def test_byte_or_string_passphrase(self):
         encoded1 = self.ref_private.export_key(format="PEM",
@@ -677,7 +773,7 @@ class TestExport_P256(unittest.TestCase):
                                           use_pkcs8=False,
                                           passphrase=b"secret",
                                           randfunc=get_fixed_prng())
-        self.assertEquals(encoded1, encoded2)
+        self.assertEqual(encoded1, encoded2)
 
     def test_error_params1(self):
         # Unknown format
@@ -752,7 +848,9 @@ gVnJp9EBND/tHQ==
 
 class TestExport_P384(unittest.TestCase):
 
-    ref_private, ref_public = create_ref_keys_p384()
+    def __init__(self, *args, **kwargs):
+        super(TestExport_P384, self).__init__(*args, **kwargs)
+        self.ref_private, self.ref_public = create_ref_keys_p384()
 
     def test_export_public_der_uncompressed(self):
         key_file = load_file("ecc_p384_public.der")
@@ -774,6 +872,21 @@ class TestExport_P384(unittest.TestCase):
         key_file_compressed_ref = load_file("ecc_p384_public_compressed.der")
         self.assertEqual(key_file_compressed, key_file_compressed_ref)
 
+    def test_export_public_sec1_uncompressed(self):
+        key_file = load_file("ecc_p384_public.der")
+        value = extract_bitstring_from_spki(key_file)
+
+        encoded = self.ref_public.export_key(format="SEC1")
+        self.assertEqual(value, encoded)
+
+    def test_export_public_sec1_compressed(self):
+        key_file = load_file("ecc_p384_public.der")
+        encoded = self.ref_public.export_key(format="SEC1", compress=True)
+
+        key_file_compressed_ref = load_file("ecc_p384_public_compressed.der")
+        value = extract_bitstring_from_spki(key_file_compressed_ref)
+        self.assertEqual(value, encoded)
+
     def test_export_private_der(self):
         key_file = load_file("ecc_p384_private.der")
 
@@ -904,15 +1017,15 @@ class TestExport_P384(unittest.TestCase):
         key_file = load_file("ecc_p384_public_openssh.txt", "rt")
 
         encoded = self.ref_public._export_openssh(False)
-        self.assertEquals(key_file, encoded)
+        self.assertEqual(key_file, encoded)
 
         # ---
 
         encoded = self.ref_public.export_key(format="OpenSSH")
-        self.assertEquals(key_file, encoded)
+        self.assertEqual(key_file, encoded)
 
         encoded = self.ref_public.export_key(format="OpenSSH", compress=False)
-        self.assertEquals(key_file, encoded)
+        self.assertEqual(key_file, encoded)
 
     def test_export_openssh_compressed(self):
         key_file = load_file("ecc_p384_public_openssh.txt", "rt")
@@ -920,7 +1033,7 @@ class TestExport_P384(unittest.TestCase):
 
         key_file_compressed = pub_key.export_key(format="OpenSSH", compress=True)
         assert len(key_file) > len(key_file_compressed)
-        self.assertEquals(pub_key, ECC.import_key(key_file_compressed))
+        self.assertEqual(pub_key, ECC.import_key(key_file_compressed))
 
     def test_prng(self):
         # Test that password-protected containers use the provided PRNG
@@ -932,7 +1045,7 @@ class TestExport_P384(unittest.TestCase):
                                           passphrase="secret",
                                           protection="PBKDF2WithHMAC-SHA1AndAES128-CBC",
                                           randfunc=get_fixed_prng())
-        self.assertEquals(encoded1, encoded2)
+        self.assertEqual(encoded1, encoded2)
 
         # ---
 
@@ -944,7 +1057,7 @@ class TestExport_P384(unittest.TestCase):
                                           use_pkcs8=False,
                                           passphrase="secret",
                                           randfunc=get_fixed_prng())
-        self.assertEquals(encoded1, encoded2)
+        self.assertEqual(encoded1, encoded2)
 
     def test_byte_or_string_passphrase(self):
         encoded1 = self.ref_private.export_key(format="PEM",
@@ -955,7 +1068,7 @@ class TestExport_P384(unittest.TestCase):
                                           use_pkcs8=False,
                                           passphrase=b"secret",
                                           randfunc=get_fixed_prng())
-        self.assertEquals(encoded1, encoded2)
+        self.assertEqual(encoded1, encoded2)
 
     def test_error_params1(self):
         # Unknown format
@@ -1019,7 +1132,9 @@ YC46ZRsnKNayw3wATdPjgja7L/DSII3nZK0G6KOOVwJBznT/e+zudUJYhZKaBLRx
 
 class TestExport_P521(unittest.TestCase):
 
-    ref_private, ref_public = create_ref_keys_p521()
+    def __init__(self, *args, **kwargs):
+        super(TestExport_P521, self).__init__(*args, **kwargs)
+        self.ref_private, self.ref_public = create_ref_keys_p521()
 
     def test_export_public_der_uncompressed(self):
         key_file = load_file("ecc_p521_public.der")
@@ -1041,6 +1156,21 @@ class TestExport_P521(unittest.TestCase):
         key_file_compressed_ref = load_file("ecc_p521_public_compressed.der")
         self.assertEqual(key_file_compressed, key_file_compressed_ref)
 
+    def test_export_public_sec1_uncompressed(self):
+        key_file = load_file("ecc_p521_public.der")
+        value = extract_bitstring_from_spki(key_file)
+
+        encoded = self.ref_public.export_key(format="SEC1")
+        self.assertEqual(value, encoded)
+
+    def test_export_public_sec1_compressed(self):
+        key_file = load_file("ecc_p521_public.der")
+        encoded = self.ref_public.export_key(format="SEC1", compress=True)
+
+        key_file_compressed_ref = load_file("ecc_p521_public_compressed.der")
+        value = extract_bitstring_from_spki(key_file_compressed_ref)
+        self.assertEqual(value, encoded)
+
     def test_export_private_der(self):
         key_file = load_file("ecc_p521_private.der")
 
@@ -1171,15 +1301,15 @@ class TestExport_P521(unittest.TestCase):
         key_file = load_file("ecc_p521_public_openssh.txt", "rt")
 
         encoded = self.ref_public._export_openssh(False)
-        self.assertEquals(key_file, encoded)
+        self.assertEqual(key_file, encoded)
 
         # ---
 
         encoded = self.ref_public.export_key(format="OpenSSH")
-        self.assertEquals(key_file, encoded)
+        self.assertEqual(key_file, encoded)
 
         encoded = self.ref_public.export_key(format="OpenSSH", compress=False)
-        self.assertEquals(key_file, encoded)
+        self.assertEqual(key_file, encoded)
 
     def test_export_openssh_compressed(self):
         key_file = load_file("ecc_p521_public_openssh.txt", "rt")
@@ -1187,7 +1317,7 @@ class TestExport_P521(unittest.TestCase):
 
         key_file_compressed = pub_key.export_key(format="OpenSSH", compress=True)
         assert len(key_file) > len(key_file_compressed)
-        self.assertEquals(pub_key, ECC.import_key(key_file_compressed))
+        self.assertEqual(pub_key, ECC.import_key(key_file_compressed))
 
     def test_prng(self):
         # Test that password-protected containers use the provided PRNG
@@ -1199,7 +1329,7 @@ class TestExport_P521(unittest.TestCase):
                                           passphrase="secret",
                                           protection="PBKDF2WithHMAC-SHA1AndAES128-CBC",
                                           randfunc=get_fixed_prng())
-        self.assertEquals(encoded1, encoded2)
+        self.assertEqual(encoded1, encoded2)
 
         # ---
 
@@ -1211,7 +1341,7 @@ class TestExport_P521(unittest.TestCase):
                                           use_pkcs8=False,
                                           passphrase="secret",
                                           randfunc=get_fixed_prng())
-        self.assertEquals(encoded1, encoded2)
+        self.assertEqual(encoded1, encoded2)
 
     def test_byte_or_string_passphrase(self):
         encoded1 = self.ref_private.export_key(format="PEM",
@@ -1222,7 +1352,7 @@ class TestExport_P521(unittest.TestCase):
                                           use_pkcs8=False,
                                           passphrase=b"secret",
                                           randfunc=get_fixed_prng())
-        self.assertEquals(encoded1, encoded2)
+        self.assertEqual(encoded1, encoded2)
 
     def test_error_params1(self):
         # Unknown format
@@ -1289,14 +1419,18 @@ vv6oYkMIIi7r5oQWAiQDrR2mlrrFDL9V7GH/r8SWQw==
 def get_tests(config={}):
     tests = []
     tests += list_test_cases(TestImport)
-    tests += list_test_cases(TestImport_P256)
-    tests += list_test_cases(TestImport_P384)
-    tests += list_test_cases(TestImport_P521)
-    tests += list_test_cases(TestExport_P256)
-    tests += list_test_cases(TestExport_P384)
-    tests += list_test_cases(TestExport_P521)
+    try:
+        tests += list_test_cases(TestImport_P256)
+        tests += list_test_cases(TestImport_P384)
+        tests += list_test_cases(TestImport_P521)
+        tests += list_test_cases(TestExport_P256)
+        tests += list_test_cases(TestExport_P384)
+        tests += list_test_cases(TestExport_P521)
+    except MissingTestVectorException:
+        pass
     return tests
 
+
 if __name__ == '__main__':
     suite = lambda: unittest.TestSuite(get_tests())
     unittest.main(defaultTest='suite')
