@@ -28,6 +28,8 @@
 # POSSIBILITY OF SUCH DAMAGE.
 # ===================================================================
 
+import os
+import abc
 import sys
 from Cryptodome.Util.py3compat import byte_string
 from Cryptodome.Util._file_system import pycryptodome_filename
@@ -35,8 +37,7 @@ from Cryptodome.Util._file_system import pycryptodome_filename
 #
 # List of file suffixes for Python extensions
 #
-if sys.version_info[0] < 3 or \
-   (sys.version_info[0] == 3 and sys.version_info[1] <= 3):
+if sys.version_info[0] < 3:
 
     import imp
     extension_suffixes = []
@@ -50,12 +51,31 @@ else:
     extension_suffixes = machinery.EXTENSION_SUFFIXES
 
 # Which types with buffer interface we support (apart from byte strings)
-if sys.version_info[0] == 2 and sys.version_info[1] < 7:
-    _buffer_type = (bytearray)
-else:
-    _buffer_type = (bytearray, memoryview)
+_buffer_type = (bytearray, memoryview)
+
+
+class _VoidPointer(object):
+    @abc.abstractmethod
+    def get(self):
+        """Return the memory location we point to"""
+        return
+
+    @abc.abstractmethod
+    def address_of(self):
+        """Return a raw pointer to this pointer"""
+        return
+
 
 try:
+    # Starting from v2.18, pycparser (used by cffi for in-line ABI mode)
+    # stops working correctly when PYOPTIMIZE==2 or the parameter -OO is
+    # passed. In that case, we fall back to ctypes.
+    # Note that PyPy ships with an old version of pycparser so we can keep
+    # using cffi there.
+    # See https://github.com/Legrandin/pycryptodome/issues/228
+    if '__pypy__' not in sys.builtin_module_names and sys.flags.optimize == 2:
+        raise ImportError("CFFI with optimize=2 fails due to pycparser bug.")
+
     from cffi import FFI
 
     ffi = FFI()
@@ -73,7 +93,10 @@ try:
         @cdecl, the C function declarations.
         """
 
-        lib = ffi.dlopen(name)
+        if hasattr(ffi, "RTLD_DEEPBIND") and not os.getenv('PYCRYPTODOME_DISABLE_DEEPBIND'):
+            lib = ffi.dlopen(name, ffi.RTLD_DEEPBIND)
+        else:
+            lib = ffi.dlopen(name)
         ffi.cdef(cdecl)
         return lib
 
@@ -82,14 +105,25 @@ try:
         return x
 
     c_ulonglong = c_ulong
+    c_uint = c_ulong
+    c_ubyte = c_ulong
 
     def c_size_t(x):
         """Convert a Python integer to size_t"""
         return x
 
-    def create_string_buffer(size):
+    def create_string_buffer(init_or_size, size=None):
         """Allocate the given amount of bytes (initially set to 0)"""
-        return ffi.new("uint8_t[]", size)
+
+        if isinstance(init_or_size, bytes):
+            size = max(len(init_or_size) + 1, size)
+            result = ffi.new("uint8_t[]", size)
+            result[:] = init_or_size
+        else:
+            if size:
+                raise ValueError("Size must be specified once only")
+            result = ffi.new("uint8_t[]", init_or_size)
+        return result
 
     def get_c_string(c_string):
         """Convert a C string into a Python byte sequence"""
@@ -108,7 +142,7 @@ try:
         else:
             raise TypeError("Object type %s cannot be passed to C code" % type(data))
 
-    class VoidPointer(object):
+    class VoidPointer_cffi(_VoidPointer):
         """Model a newly allocated pointer to void"""
 
         def __init__(self):
@@ -120,21 +154,34 @@ try:
         def address_of(self):
             return self._pp
 
+    def VoidPointer():
+        return VoidPointer_cffi()
+
     backend = "cffi"
 
 except ImportError:
 
     import ctypes
     from ctypes import (CDLL, c_void_p, byref, c_ulong, c_ulonglong, c_size_t,
-                        create_string_buffer, c_ubyte)
+                        create_string_buffer, c_ubyte, c_uint)
     from ctypes.util import find_library
-    from _ctypes import Array as _Array
+    from ctypes import Array as _Array
 
     null_pointer = None
+    cached_architecture = []
+
+    def c_ubyte(c):
+        if not (0 <= c < 256):
+            raise OverflowError()
+        return ctypes.c_ubyte(c)
 
     def load_lib(name, cdecl):
-        import platform
-        bits, linkage = platform.architecture()
+        if not cached_architecture:
+            # platform.architecture() creates a subprocess, so caching the
+            # result makes successive imports faster.
+            import platform
+            cached_architecture[:] = platform.architecture()
+        bits, linkage = cached_architecture
         if "." not in name and not linkage.startswith("Win"):
             full_name = find_library(name)
             if full_name is None:
@@ -150,20 +197,16 @@ except ImportError:
 
     # ---- Get raw pointer ---
 
-    if sys.version_info[0] == 2 and sys.version_info[1] == 6:
-        # ctypes in 2.6 does not define c_ssize_t. Replacing it
-        # with c_size_t keeps the structure correctely laid out
-        _c_ssize_t = c_size_t
-    else:
-        _c_ssize_t = ctypes.c_ssize_t
+    _c_ssize_t = ctypes.c_ssize_t
 
     _PyBUF_SIMPLE = 0
     _PyObject_GetBuffer = ctypes.pythonapi.PyObject_GetBuffer
+    _PyBuffer_Release = ctypes.pythonapi.PyBuffer_Release
     _py_object = ctypes.py_object
     _c_ssize_p = ctypes.POINTER(_c_ssize_t)
 
     # See Include/object.h for CPython
-    # and https://github.com/pallets/click/blob/master/click/_winconsole.py
+    # and https://github.com/pallets/click/blob/master/src/click/_winconsole.py
     class _Py_buffer(ctypes.Structure):
         _fields_ = [
             ('buf',         c_void_p),
@@ -180,7 +223,7 @@ except ImportError:
         ]
 
         # Extra field for CPython 2.6/2.7
-        if sys.version_info[0] == 2 or (sys.version_info[0] == 3 and sys.version_info[1] <= 2):
+        if sys.version_info[0] == 2:
             _fields_.insert(-1, ('smalltable', _c_ssize_t * 2))
 
     def c_uint8_ptr(data):
@@ -190,14 +233,17 @@ except ImportError:
             obj = _py_object(data)
             buf = _Py_buffer()
             _PyObject_GetBuffer(obj, byref(buf), _PyBUF_SIMPLE)
-            buffer_type = c_ubyte * buf.len
-            return buffer_type.from_address(buf.buf)
+            try:
+                buffer_type = ctypes.c_ubyte * buf.len
+                return buffer_type.from_address(buf.buf)
+            finally:
+                _PyBuffer_Release(byref(buf))
         else:
             raise TypeError("Object type %s cannot be passed to C code" % type(data))
 
     # ---
 
-    class VoidPointer(object):
+    class VoidPointer_ctypes(_VoidPointer):
         """Model a newly allocated pointer to void"""
 
         def __init__(self):
@@ -209,8 +255,10 @@ except ImportError:
         def address_of(self):
             return byref(self._p)
 
+    def VoidPointer():
+        return VoidPointer_ctypes()
+
     backend = "ctypes"
-    del ctypes
 
 
 class SmartPointer(object):
@@ -251,11 +299,21 @@ def load_pycryptodome_raw_lib(name, cdecl):
     for ext in extension_suffixes:
         try:
             filename = basename + ext
-            return load_lib(pycryptodome_filename(dir_comps, filename),
-                            cdecl)
+            full_name = pycryptodome_filename(dir_comps, filename)
+            if not os.path.isfile(full_name):
+                attempts.append("Not found '%s'" % filename)
+                continue
+            return load_lib(full_name, cdecl)
         except OSError as exp:
-            attempts.append("Trying '%s': %s" % (filename, str(exp)))
+            attempts.append("Cannot load '%s': %s" % (filename, str(exp)))
     raise OSError("Cannot load native module '%s': %s" % (name, ", ".join(attempts)))
 
-def expect_byte_string(data):
-    raise NotImplementedError("To be removed")
+
+def is_buffer(x):
+    """Return True if object x supports the buffer interface"""
+    return isinstance(x, (bytes, bytearray, memoryview))
+
+
+def is_writeable_buffer(x):
+    return (isinstance(x, bytearray) or
+            (isinstance(x, memoryview) and not x.readonly))
